import { DrawPile } from "./drawpile.ts";
import { Point, Rect, Size } from "./engine/datatypes.ts";
import { getPartLocation, withCamera } from "./layout.ts";
import { addButton } from "./button.ts";
import { D } from "./engine/public.ts";
import { BG_INSET } from "./colors.ts";
import { getSkillsModal } from "./skillsmodal.ts";
import { getStateManager } from "./statemanager.ts";

export class SleepModal {
  #drawpile: DrawPile;
  #shown: boolean;

  constructor() {
    this.#drawpile = new DrawPile();
    this.#shown = false;
  }

  get #size(): Size {
    // We share this logic with SkillModal:
    // Instead of calculating this here, compute it from outside
    // as it has to be the same for every bottom modal
    return getPartLocation("BottomModal").size;
  }

  get isShown(): boolean {
    return this.#shown;
  }

  setShown(shown: boolean) {
    this.#shown = shown;
  }

  update() {
    withCamera("BottomModal", () => this.#update());
  }

  draw() {
    withCamera("BottomModal", () => this.#draw());
  }

  #update() {
    this.#drawpile.clear();
    let size = this.#size;
    this.#drawpile.add(0, () => {
      D.fillRect(new Point(-4, -4), size.add(new Size(8, 8)), BG_INSET);
    });

    // add close button
    let closeRect = new Rect(new Point(0, 96), new Size(80, 32));
    addButton(this.#drawpile, "Back", closeRect, true, () => {
      this.setShown(false);
    });

    let skillsRect = new Rect(new Point(80, 96), new Size(80, 32));
    addButton(this.#drawpile, "Skills", skillsRect, true, () => {
      getSkillsModal().setShown(true);
    });

    let remainingWidth = size.w - 160;
    let nextRect = new Rect(new Point(160, 96), new Size(remainingWidth, 32));
    addButton(this.#drawpile, "Sleep (Next Day)", nextRect, true, () => {
      getStateManager().advance();
    });

    this.#drawpile.executeOnClick();
  }

  #draw() {
    this.#drawpile.draw();
  }
}

let active = new SleepModal();
export function getSleepModal(): SleepModal {
  return active;
}
